//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:33 $
//
//==============================================================================
//
// Class: ArrayAutoPtr
// 
/**
	@class ot::ArrayAutoPtr

    @brief A variation of @c std::auto_ptr which holds a pointer to an array
           and uses the associated @c delete [] syntax when it needs to destroy
		   the array.

	The following example shows how an ArrayAutoPtr is initialized with
	a new array pointer and manages the memory even in the presence of
	exceptions:

    @code
    // create a scope for the automatic deletion of the array
    {
        ArrayAutoPtr<float> apArray(new float[20]);
        // use operator[] to access the array members
        apArray[0] = 3.142;
        throw Exception(OT_T("test"));
        // array is automatically freed at exit from scope
    }
    @endcode

    @sa std::auto_ptr
*/
//==============================================================================

#ifndef OT_BASE_defs_h
#include "defs.h"
#endif //OT_BASE_defs_h

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
/**
   Creates an ArrayAutoPtr which takes ownership of the array 
   starting at @c ptr.
*/
//==============================================================================
template<typename T>
inline ArrayAutoPtr<T>::ArrayAutoPtr(T* ptr) :
	m_bOwner(ptr != 0), m_ptr(ptr)
{}

//==============================================================================
/**
   Copy constructor which takes ownership of the array currently
   owned by @c rhs.

   Note that @c rhs is modified by this operation
   even though it is marked as @c const.
*/
//==============================================================================
template<typename T>
inline ArrayAutoPtr<T>::ArrayAutoPtr(const ArrayAutoPtr<T>& rhs) :
	m_bOwner(rhs.m_bOwner), m_ptr(rhs.release())
{
}

//==============================================================================
/**
   Assignment operator which takes ownership of the array currently
   owned by @c rhs.  If this ArrayAutoPtr already owns an
   array pointer it is freed.

   Note that @c rhs is modified by this operation
   even though it is marked as @c const.
*/
//==============================================================================
template<typename T>
inline ArrayAutoPtr<T>& ArrayAutoPtr<T>::operator=(const ArrayAutoPtr<T>& rhs)
{
	if (this != &rhs)
	{
		if (m_ptr != rhs.get())
		{
			if (m_bOwner)
				delete [] m_ptr;
			m_bOwner = rhs.m_bOwner;
		}
		else if (rhs.m_bOwner)
			m_bOwner = true;
		m_ptr = rhs.release();
	}
	return (*this);
}

//==============================================================================
/**
   Destructor which deletes the contained array pointer if it is 'owned'
   by this object.
*/
//==============================================================================
template<typename T>
inline ArrayAutoPtr<T>::~ArrayAutoPtr()
{
	if (m_bOwner)
		delete [] m_ptr;
}

//==============================================================================
/**
   Dereference operator which returns a reference to the first element of the
   contained array.
*/
//==============================================================================
template<typename T>
inline T& ArrayAutoPtr<T>::operator*() const 
{
	return (*get());
}

//==============================================================================
/**
   Dereference operator which returns a reference to the nth element of the
   contained array.
*/
//==============================================================================
template<typename T>
inline T& ArrayAutoPtr<T>::operator[](size_t n) const 
{
	return (*(m_ptr+n));
}

//==============================================================================
/**
   Returns the contained array pointer.
*/
//==============================================================================
template<typename T>
inline T* ArrayAutoPtr<T>::get() const 
{
	return (m_ptr);
}

//==============================================================================
/**
   Returns the contained pointer and revokes 'ownership', so that the
   destructor or assignment will not delete the contained pointer.
*/
//==============================================================================
template<typename T>
inline T* ArrayAutoPtr<T>::release() const 
{
	// handle lack of mutable support
	((ArrayAutoPtr<T>*)this)->m_bOwner = false;
	return (m_ptr);
}

OT_BASE_NAMESPACE_END
