//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.2 $
// $Date: 2003/08/28 22:48:46 $
//
//==============================================================================
//
// Class AutoBuffer
//
/**
	@class ot::AutoBuffer
	
	@brief An efficient buffer that will automatically grow to hold the
	required data.
	
    AutoBuffer is often used internally within @OpenTop when building
	String objects.

    In the following example an AutoBuffer is created to hold a string
	containing all the Unicode characters from U+0000 - U+1000.
    @code
    AutoBuffer<CharType> charBuffer;
    for(UCS4Char cp=0; cp<0x1000; ++cp)
    {
        // convert the Unicode code-point into a Character
        Character ch(cp);	
        // append the CharType sequence from ch onto our buffer
        charBuffer.append(ch.data(), ch.length());
    }
    @endcode
*/
//==============================================================================


#include "defs.h"
#include "debug.h"

#include <memory.h>

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// AutoBuffer<T>::AutoBuffer()
//
/**
   Default constructor which creates an internal buffer with a default initial
   size.
*/
//==============================================================================
template<typename T>
inline
AutoBuffer<T>::AutoBuffer()
{
	const int defaultSize=240;
	init(defaultSize);
}
	
//==============================================================================
// AutoBuffer<T>::AutoBuffer(size_t initialSize)
//
/**
   Default constructor which creates an internal buffer with an initial
   size of @c initialSize.
   
   @param initialSize the initial size of the internal buffer.
*/
//==============================================================================
template<typename T>
inline
AutoBuffer<T>::AutoBuffer(size_t initialSize)
{
	init(initialSize);
}

//==============================================================================
// AutoBuffer<T>::~AutoBuffer()
//
/**
   Destructor.  The internal buffer is freed.
*/
//==============================================================================
template<typename T>
inline
AutoBuffer<T>::~AutoBuffer()
{
	delete [] m_pBuffer;
}

//==============================================================================
// AutoBuffer<T>::clear()
//
/**
   Clears the internal buffer for re-use - the memory is not freed.
*/
//==============================================================================
template<typename T>
inline
void AutoBuffer<T>::clear()
{
	m_used = 0;
}	

//==============================================================================
// AutoBuffer<T>::destroy()
//
/**
   Frees the internal buffer.  Subsequent append() operations will cause a new
   buffer to be allocated.
*/
//==============================================================================
template<typename T>
inline
void AutoBuffer<T>::destroy()
{
	delete [] m_pBuffer;
	m_pBuffer = m_used = m_size = 0;
}

//==============================================================================
// AutoBuffer<T>::size()
//
/**
   Returns the number of buffer elements currently in use.  This is not the
   same as the capacity of the buffer.
   
   @sa capacity()
*/
//==============================================================================
template<typename T>
inline
size_t AutoBuffer<T>::size() const
{
	return m_used;
}

//==============================================================================
// AutoBuffer<T>::capacity()
//
/**
   Returns the allocated size of the internal buffer.  This is not necessarily
   the number of buffer elements that are currently being used.
   
   @sa size()
*/
//==============================================================================
template<typename T>
inline
size_t AutoBuffer<T>::capacity() const
{
	return m_size;
}

//==============================================================================
// AutoBuffer<T>::data()
//
/**
   Returns a const pointer to the start of the internal buffer.
*/
//==============================================================================
template<typename T>
inline
const T* AutoBuffer<T>::data() const
{
	return m_pBuffer;
}

//==============================================================================
// AutoBuffer<T>::append()
//
/**
   Appends a single element of type T to the internal buffer.  The buffer is
   re-sized if necessary.

   @param data the element to append to the internal buffer
*/
//==============================================================================
template<typename T>
inline
void AutoBuffer<T>::append(T data)
{
	append(&data, 1);
}

//==============================================================================
// AutoBuffer<T>::append()
//
/**
   Appends an array of elements of type T to the internal buffer.  The buffer is
   re-sized if necessary.

   @param pData a pointer to the first element in the array
   @param length the number of elements to append
*/
//==============================================================================
template<typename T>
inline
void AutoBuffer<T>::append(const T* pData, size_t length)
{
	if(length > 0)
	{
		size_t freeSize = (m_size - m_used);
		if(freeSize < length)
		{
			//
			// Decide how big to make the new buffer.
			// We need a compromise between continually re-sizing the buffer,
			// causing memory fragmentation and memory starvation.
			//
			enum {MinSize = sizeof(DataType) <= 32 ? 31 : 7};
			size_t growth = (m_used + length) / 2;
			m_size = m_used + length + growth;
			if(m_size < MinSize) m_size = MinSize;

			DataType* pNewBuffer = new DataType[m_size];
			if(m_pBuffer)
			{
				if(m_used)
				{
					::memcpy((void*)pNewBuffer, (void*)m_pBuffer, m_used*sizeof(DataType));
				}
				delete [] m_pBuffer;
			}
			m_pBuffer = pNewBuffer;
		}

		OT_DBG_ASSERT(m_pBuffer!=0);
		OT_DBG_ASSERT(length <= (m_size - m_used));

		::memcpy(m_pBuffer+m_used, pData, length*sizeof(DataType));
		m_used += length;
	}
}

//==============================================================================
// AutoBuffer<T>::init
//
// Private initialization functions
//==============================================================================
template<typename T>
inline
void AutoBuffer<T>::init(size_t bufferSize)
{
	m_pBuffer=0;
	m_used=0;
	m_size=bufferSize;

	if(m_size)
	{
		m_pBuffer = new DataType[m_size];
	}
}

OT_BASE_NAMESPACE_END
