//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:34 $
//
//==============================================================================
//
// Class: AutoLock
// 
/**
	@class ot::AutoLock
	
	@brief Acquires a lock on initialization and releases it on destruction.
	
    AutoLock is based on the principle described by Bjarne Stroustrup as 
	"resource acquisition is initialization".  It is designed to lock and
	unlock synchronization objects in an exception-safe fashion by using
	local objects.

    In normal use, the constructor acquires a lock.  This lock is then
	held until the object is destroyed at the end of the containing scope.

    @code
    bool Foo::synchronizedMethod()
    {
        AutoLock<Mutex> lock(m_mutex); // acquires a lock on m_mutex
        ...                            // state can be updated here
        return true;                   // m_mutex is unlocked on exit
    }
    @endcode

    The lock can be released (and re-acquired) at any time.  The general rule
	is that if the lock is still held when an AutoLock object is destroyed, it will
	automatically release the lock.

	AutoLock is a template class.  It requires classes of type T to expose just 
	two public methods: T::lock() and T::unlock().
*/
//==============================================================================

#include "defs.h"
#include "NullPointerException.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// AutoLock<T>::AutoLock(T& lock)
//
/**
   Constructor which takes a reference to @c lock and locks it.
   @param lock a reference to the synchronization object that will be managed
          by this AutoLock object.
   @throws NullPointerException if @c lock is a reference to a null pointer
*/
//==============================================================================
template<typename T>
inline
AutoLock<T>::AutoLock(T& lock) :
	m_lock(lock),
	m_bLocked(false)
{
	if(&lock == NULL) throw NullPointerException();

	m_lock.lock();
	m_bLocked = true;
}

//==============================================================================
// AutoLock<T>::AutoLock
//
/**
   Constructor which takes a reference to @c lock and locks it
   if @c bInitialLock is true.
   @param lock a reference to the synchronization object that will be managed
          by this AutoLock object.
   @param bInitialLock if true, the lock is acquired by the constructor
   @throws NullPointerException if @c lock is a reference to a null pointer
*/
//==============================================================================
template<typename T>
inline
AutoLock<T>::AutoLock(T& lock, bool bInitialLock) :
	m_lock(lock),
	m_bLocked(false)
{
	if(&lock == NULL) throw NullPointerException();

	if(bInitialLock)
	{
		m_lock.lock();
		m_bLocked = true;
	}
}

//==============================================================================
// AutoLock<T>::~AutoLock
//
/**
   Destructor which releases the lock if it is still being held.
*/
//==============================================================================
template<typename T>
inline
AutoLock<T>::~AutoLock()
{
	if(m_bLocked)
	{
		m_lock.unlock();
	}
}

//==============================================================================
// AutoLock<T>::unlock
//
/**
   Releases the lock.  If the lock is not currently being held, this method
   has no effect.
*/
//==============================================================================
template<typename T>
inline
void AutoLock<T>::unlock()
{
	if(m_bLocked)
	{
		m_lock.unlock();
		m_bLocked = false;
	}
}

//==============================================================================
// AutoLock<T>::lock
//
/**
   Acquires the lock.  If the lock is already held this method has no effect.
*/
//==============================================================================
template<typename T>
inline
void AutoLock<T>::lock()
{
	if(!m_bLocked)
	{
		m_lock.lock();
		m_bLocked = true;
	}
}

OT_BASE_NAMESPACE_END
