//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:34 $
//
//==============================================================================
//
// Class: AutoUnlock
// 
/**
	@class ot::AutoUnlock
	
	@brief Releases a lock on initialization and re-acquires it on destruction.
	
    AutoUnlock is based on the principle described by Bjarne Stroustrup as 
	"resource acquisition is initialization".  It is designed to unlock and
	re-lock synchronization objects in an exception-safe fashion by using
	local objects.

    In normal use, the constructor releases a lock.  This lock is then
	free until the object is destroyed at the end of the current scope, when it is 
	re-acquired

    @code
    bool Foo::waitMethod()
    {
        AutoUnlock<Mutex> lock(m_mutex); // releases the lock on m_mutex
        ...                              // activity that does not require the lock
        return true;                     // m_mutex is re-locked on exit
    }
    @endcode

    The lock can be re-acquired (and released) at any time.  The general rule
	is that if the lock is still not held when an AutoUnlock object is destroyed, it will
	automatically re-acquire the lock.

	AutoUnlock is a template class.  It requires classes of type T to expose just 
	two public methods: T::lock() and T::unlock().
*/
//==============================================================================

#include "defs.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// AutoUnlock<T>::AutoUnlock(T& lock)
//
/**
   Constructor which takes a reference to @c lock and unlocks it.
   @param lock a reference to the synchronization object that will be managed
          by this AutoUnlock object.
*/
//==============================================================================
template<typename T>
inline
AutoUnlock<T>::AutoUnlock(T& lock) :
	m_lock(lock),
	m_bUnlocked(false)
{
	m_lock.unlock();
	m_bUnlocked = true;
}

//==============================================================================
// AutoUnlock<T>::~AutoUnlock
//
/**
   Destructor which re-acquires the lock if it is not being held.
*/
//==============================================================================
template<typename T>
inline
AutoUnlock<T>::~AutoUnlock()
{
	if(m_bUnlocked)
	{
		m_lock.lock();
	}
}

//==============================================================================
// AutoUnlock<T>::lock
//
/**
   Re-acquires the lock if it is not being held; has no effect otherwise.
*/
//==============================================================================
template<typename T>
inline
void AutoUnlock<T>::lock()
{
	if(m_bUnlocked)
	{
		m_lock.lock();
		m_bUnlocked = false;
	}
}

//==============================================================================
// AutoUnlock<T>::unlock
//
/**
   Releases the lock if it is being held; has no effect otherwise.
*/
//==============================================================================
template<typename T>
inline
void AutoUnlock<T>::unlock()
{
	if(!m_bUnlocked)
	{
		m_lock.unlock();
		m_bUnlocked = true;
	}
}

OT_BASE_NAMESPACE_END
