//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.5 $
// $Date: 2003/08/28 19:49:05 $
//
//==============================================================================
//
// Class: RefPtr<T>
//
/**
	@class ot:: RefPtr

	Template class that helps to implement the ManagedObject
	reference-counting scheme.

	<p>RefPtr is a template class used to help implement the
	reference-counting scheme for instances of ManagedObject.  ManagedObject
	instances can be used without the help of a RefPtr handle, but using RefPtr
	helps to alleviate most of the tedious and potentially error-prone 
	reference-counting tasks.</p>

	<p>When a RefPtr is constructed or assigned, it automatically increments
	the reference count of its associated object by calling ManagedObject::addRef().
	When a RefPtr is destroyed, its destructor automatically decrements its 
	associated object's reference count by calling ManagedObject::release().</p>

	<p>The ManagedObject reference-counting scheme provides shared ownership 
	semantics, where the lifetime of the object is guaranteed to last
	until all the references to it have been destroyed.  It is safe to pass
	RefPtrs to functions and to store them within standard C++ collections.</p>

	The RefPtr class has a user-defined @c operator->() which allows it to be used
	as if it were a raw C++ pointer.  Here's a simple example:

    @code
    RefPtr<InputStream> rpInputStream = new FileInputStream(OT_T("foo.txt"));
    RefPtr<Reader> rpReader = new InputStreamReader(rpInputStream.get());
    IntType x = rpReader->read();
    @endcode

	In the example, all reference-counting is managed by RefPtrs, and the
	pointer operator was used to access the read() function.  The
	@a rp prefix is a convention used throughout @OpenTop when declaring RefPtr
	variables.

	RefPtr has been modelled so that it can be used in a similar way to a raw
	C++ pointer, with the important exception that there is no implicit conversion
	from a RefPtr to a raw pointer.  There is an automatic conversion to type @c bool
	which allows for the common practice of testing a null pointer like so:

    @code
    RefPtr<Thread> rpThisThread = Thread::CurrentThread();
    if(rpThisThread) {
        // This thread was created under the control of a Thread object
        ...
    }
    @endcode

	<h4>When should RefPtr<T> be used?</h4>

	<strong>Function return values</strong>
	<p>Functions wishing to return a pointer to a ManagedObject should instead return 
	a RefPtr.  This has the advantage that the calling function does not need
	to worry about the reference count, it is @a exception-safe 
	and can be particularly advantageous if the calling function simply wants to pass
	the result as a parameter to another function like this:</p>
	
    @code
    String sHost = OT_T("www.elcel.com");
    RefPtr<Socket> rpSocket = new Socket(InetAddress::GetByName(sHost).get(), 1234);
    @endcode

	<p>In the above code fragment, the Socket constructor was passed a pointer to an
	InetAddress which was returned from InetAddress::GetByName().
	Actually, InetAddress::GetByName() returns a RefPtr<InetAddress> which
	ensures the reference remains valid for the scope of the statement.
	The RefPtr<InetAddress> is @a explicitly converted into a raw pointer
	by the @c RefPtr::get() method.</p>

	<p>Another advantage of returning a RefPtr from a function is that function
	calls can be chained together.  For example:</p>

    @code
    String sName = rpSocket->getLocalAddress()->getHostName();
    @endcode

	<p>In the above code fragment, getLocalAddress returned a RefPtr<InetAddress>
	which was used <em>in-situ</em> to call getHostAddress().  The RefPtr<> ensures
	that the reference count of the InetAddress is properly maintained without
	adding complexity to the application.</p>

	<strong>Function input/output parameters</strong>
	<p>Function @a input parameters are not required to use RefPtrs; raw C++ pointers are
	generally more efficient and perfectly adequate in this case. Function @a output
	parameters, however, should be declared as non-const RefPtrs by reference.  For example,
	the following function makes use of return parameters to connect and return 2 
	network sockets:-</p>

    @code
    bool getSockets(const String& inStr, RefPtr<Socket>& rpOutSock1, RefPtr<Socket>& rpOutSock2)
    {
        bool bRet = true;
        try
        {
            rpOutSock1 = new Socket(inStr, 123);
            rpOutSock2 = new Socket(inStr, 456);
        }
        catch(SocketException& e)
        {
            bRet = false;
            rpOutSock1.release();
            rpOutSock2.release();
        }
        return bRet;
    }
    @endcode

	<strong>Variables</strong>
	<p>Generally, the only place where raw pointers are used is in function input 
	parameters.  Elsewhere, such as stack variables and class members,
	RefPtrs should be used instead.  This has the 
	advantage that custom destructors are not required and reference counting
	is automatic.  For example, here is the definition for a class that holds
	a counted reference to an InputStream object:</p>

    @code
    class foo
    {
    public:
        foo(InputStream* pInputStream) : m_rpInputStream(pInputStream) {}
    private:
        RefPtr<InputStream> m_rpInputStream;
    };
    @endcode

	<p>In this example the class @c foo has no need for a custom destructor, the
	reference count of the contained InputStream will be managed by the RefPtr member.</p>

	<p>Note, however, that if an object instance ever tried to store a reference to
	itself it would create a tight cyclic reference with the result
	that the reference count would never be decremented to zero.  If this is a
	possibility, use the @c RefPtrMember<T> class which has been customized to detect
	and avoid this behaviour.</p>


	<h4>Automatic conversion</h4>

	After much deliberation, the OpenTop designers decided against providing RefPtr with
	a conversion operator to allow it to be implicitly converted into a raw C++ pointer.
	Providing such a conversion operator would have made the syntax for passing RefPtrs 
	to functions simpler and more elegant, but it would also have allowed RefPtrs to be
	assigned to raw C++ pointers, a trap that inexperienced developers could all-too-easily
	fall into.  For example, the following code is incorrect because it attempts to convert
	a RefPtr into a raw C++ pointer.

    @code
    // The following line will not compile...
    InetAddress* pAddr = InetAddress::GetByName(OT_T("www.elcel.com"));
    // ...but if it did, the following line would cause an error
    String sHost = pAddr->toString();
    @endcode

	If we had provided a conversion operator, the above code would have compiled
	successfully, and the program would probably crash (or worse) when the @c pAddr variable
	was dereferenced.  This is because @c InetAddress::GetByName() returns a RefPtr<InetAddress> which,
	because it is not bound to a reference, will be immediately destroyed with the result that
	the InetAddress to which it refers will be destroyed also.  Of course the correct way to code this
	example is with a local RefPtr variable:-

    @code
    RefPtr<InetAddress> rpAddr = InetAddress::GetByName(OT_T("www.elcel.com")); 
    String sHost = rpAddr->toString();
    @endcode

	
	@sa RefPtrMember
	@sa ManagedObject
*/
//==============================================================================

#include "defs.h"
#include "debug.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// RefPtr::RefPtr
//
/**
   Default constructor.

   The internal object pointer is initialized to null.
*/
//==============================================================================
template<typename T>
inline
RefPtr<T>::RefPtr() : m_ptr(0)
{
}

//==============================================================================
// RefPtr::RefPtr
//
/**
   Constructs a RefPtr with a pointer to T.

   If @c ptr is not null, addRef() is called to increment the reference count.

   @param ptr pointer to an object of type T, which is usually a class
   derived from ManagedObject.
*/
//==============================================================================
template<typename T>
inline
RefPtr<T>::RefPtr(T* ptr) : m_ptr(ptr)
{
	if(m_ptr)
		m_ptr->addRef();
}

//==============================================================================
// RefPtr::RefPtr
//
/**
   Constructs a RefPtr from another RefPtr with the same object type.

   If @c rhs contains a non-null object pointer, addRef() is called 
   to increment the reference count.

   @param rhs the RefPtr<T> to copy.
*/
//==============================================================================
template<typename T>
inline
RefPtr<T>::RefPtr(const RefPtr& rhs) : m_ptr(rhs.m_ptr)
{
	if(m_ptr)
		m_ptr->addRef();
}

//==============================================================================
// RefPtr::operator=
//
/**
   Assigns one RefPtr to another.

   If @c rhs contains a non-null object pointer, addRef() is called 
   to increment the reference count.

   @param rhs the RefPtr<T> to copy.
*/
//==============================================================================
template<typename T>
inline
RefPtr<T>& RefPtr<T>::operator=(const RefPtr<T>& rhs)
{
	if(m_ptr != rhs.get())
	{
		// do not release the current ptr until we have addrefed
		// the new value - just in case the new value relies on
		// the existing value for its continued existence
		T* pOld = m_ptr;

		m_ptr = rhs.get();
		if(m_ptr)
		{
			m_ptr->addRef();
		}
		if(pOld)
		{
			pOld->release();
		}

	}
	return *this;
}

//==============================================================================
// RefPtr::operator=
//
/**
   Assigns a pointer.

   If @c ptr is not null, addRef() is called 
   to increment the reference count.  If this RefPtr previously referenced
   an object, release() is called on that object.

   @param ptr pointer to an object of type T
*/
//==============================================================================
template<typename T>
inline
RefPtr<T>& RefPtr<T>::operator=(T* ptr)
{
	if(m_ptr != ptr)
	{
		// do not release the current ptr until we have addrefed
		// the new value - just in case the new value relies on
		// the existing value for its continued existence
		T* pOld = m_ptr;
		m_ptr = ptr;
		if(m_ptr)
		{
			m_ptr->addRef();
		}
		if(pOld)
		{
			pOld->release();
		}
	}
	return *this;
}

//==============================================================================
// RefPtr::operator<
//
/**
   Less than operator.

   Returns true if this RefPtr should be ordered before @c rhs.
   If this and @c rhs both refer to null, @c false is returned.
   @param rhs the RefPtr to compare.
   @since 1.3
*/
//==============================================================================
template<typename T>
inline
bool RefPtr<T>::operator<(const RefPtr& rhs) const
{
	return (m_ptr < rhs.m_ptr);
}

//==============================================================================
// RefPtr::operator==
//
/**
   Equality operator.

   Returns true if this RefPtr refers to the same object as @c rhs.
   If this and @c rhs both refer to null, @c true is returned.
   @param rhs the RefPtr to compare.
*/
//==============================================================================
template<typename T>
inline
bool RefPtr<T>::operator==(const RefPtr& rhs) const
{
	return (m_ptr == rhs.m_ptr);
}

//==============================================================================
// RefPtr::operator==
//
/**
   Equality operator.

   Returns true if this RefPtr refers to the same object as @c rhs.
   If this and @c rhs both refer to null, true is returned.
   @param rhs a pointer to an object of type @c T which will be compared
          with the contained pointer
*/
//==============================================================================
template<typename T>
inline
bool RefPtr<T>::operator==(const T* rhs) const
{
	return (m_ptr == rhs);
}

//==============================================================================
// RefPtr::operator!=
//
/**
   Inequality operator.

   Returns true if this RefPtr refers to a different object than @c rhs.
   If this and @c rhs both refer to null, @c false is returned.

   @param rhs the RefPtr to compare.
*/
//==============================================================================
template<typename T>
inline
bool RefPtr<T>::operator!=(const RefPtr& rhs) const
{
	return !(*this == rhs);
}

//==============================================================================
// RefPtr::operator!=
//
/**
   Inequality operator.

   Returns true if this RefPtr refers to a different object than @c rhs.
   If this and @c rhs both refer to null, @c false is returned.

   @param rhs a pointer to an object of type @c T which will be compared
          against the contained pointer
*/
//==============================================================================
template<typename T>
inline
bool RefPtr<T>::operator!=(const T* rhs) const
{
	return (m_ptr != rhs);
}

//==============================================================================
// RefPtr::operator bool
//
/**
   Conversion operator.

   @returns @c true if this RefPtr is not equal to null.
   See the class description for further information about
   using the conversion operator.
*/
//==============================================================================
template<typename T>
inline 
RefPtr<T>::operator bool() const
{
	return (m_ptr!=0);
}

//==============================================================================
// RefPtr::isNull
//
/**
   Tests if the contained object pointer is null

   @returns @c true if the object pointer is null; @c false otherwise
*/
//==============================================================================
template<typename T>
inline
bool RefPtr<T>::isNull() const
{
	return (m_ptr == 0);
}

//==============================================================================
// RefPtr::get
//
/**
   Returns the contained object pointer.

   Neither the RefPtr nor the reference count of its object are changed.
*/
//==============================================================================
template<typename T>
inline 
T* RefPtr<T>::get() const
{
	return m_ptr;
}

//==============================================================================
// RefPtr::operator->
//
/**
   A dereferencing operator.  Returns the contained object pointer.
   This operator allows RefPtrs to be used as if they were raw C++ pointers.
*/
//==============================================================================
template<typename T>
inline 
T* RefPtr<T>::operator->() const
{
	OT_DBG_ASSERT(m_ptr!=0);
	return m_ptr;
}

//==============================================================================
// RefPtr::operator*
//
/**
   A dereferencing operator.  Returns the contained object.
   This operator allows RefPtrs to be used as if they were raw C++ pointers.
*/
//==============================================================================
template<typename T>
inline 
T& RefPtr<T>::operator*() const
{
	OT_DBG_ASSERT(m_ptr!=0);
	return *m_ptr;
}

//==============================================================================
// RefPtr::~RefPtr
//
/**
   Destructor.  Decrements the reference count of the contained object pointer
   if it is not null.
*/
//==============================================================================
template<typename T>
inline 
RefPtr<T>::~RefPtr()
{
	release();
}

//==============================================================================
// RefPtr::release
//
/**
   Decrements the reference count of the contained object pointer
   if it is not null, before setting it to null.
*/
//==============================================================================
template<typename T>
inline 
void RefPtr<T>::release()
{
	if(m_ptr)
	{
		m_ptr->release();
		m_ptr = 0;
	}
}

OT_BASE_NAMESPACE_END
