//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.3 $
// $Date: 2003/08/28 19:49:05 $
//
//==============================================================================
//
// Class: RefPtrMember<T>
//
/**
   @class ot:: RefPtrMember

   Template class designed to be used as a member variable within a class
   that needs to store object references, possibly even a reference to itself.

    <h4>Overview</h4>

	The RefPtrMember class is very similar in design and operation to RefPtr<T>,
	with the exception that it will not increment or decrement the reference count
	of an object that is referring to itself.

	Use RefPtrMember in preference to RefPtr when a class contains an object
	reference as a member variable, and the possibility exists for that member
	to be set equal to the containing object.

    In the following example, the @c CoffeeFilter class uses a @c FilterEventHandler class,
	presumably to report interesting events that happen to the coffee filter.  The 
	derived class @c MyFilter wishes to implement a
	customized filter and deal with its events.  To achieve this,
	it needs to call @c setEventHandler(this), which would create a 
	cyclic reference if the reference count of the FilterEventHandler was incremented
	naively.

    @code
    class FilterEventHandler;
    class CoffeeFilter : public ManagedObject
    {
        public:
            void setEventHandler(FilterEventHandler* pHandler);

        private:
            RefPtrMember<FilterEventHandler> m_rpEventHandler;
    };

    class MyFilter : public CoffeeFilter, public FilterEventHandler
    {
        public:
            MyFilter()  {
                setEventHandler(this);
            }
    };
    @endcode

*/
//==============================================================================

#include "NullPointerException.h"
#include "debug.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// RefPtrMember::RefPtrMember
//
/**
   Standard constructor, requires a pointer to a ManagedObject which is the
   parent object containing this RefPtrMember as a member.

   The internal object pointer is initialized to null.
   @param pParent the parent ManagedObject containing this RefPtrMember as a member
*/
//==============================================================================
template<typename T>
inline
RefPtrMember<T>::RefPtrMember(ManagedObject* pParent) :
	m_ptr(0),
	m_pParent(pParent)
{
	if(!pParent) throw NullPointerException();
}

//==============================================================================
// RefPtrMember::RefPtrMember
//
/**
   Constructs a RefPtrMember with a pointer to T.

   If @c ptr is not null and is not a reference to the parent object, addRef() is called
   to increment the reference count.

   @param pParent the parent ManagedObject containing this RefPtrMember as a member
   @param ptr pointer to an object of type T, which must be a class
   derived from ManagedObject.
*/
//==============================================================================
template<typename T>
inline
RefPtrMember<T>::RefPtrMember(ManagedObject* pParent, T* ptr) :
	m_ptr(ptr),
	m_pParent(pParent)
{
	if(m_ptr && (static_cast<ManagedObject*>(m_ptr) != m_pParent))
		m_ptr->addRef();
}

//==============================================================================
// RefPtrMember::RefPtrMember
//
/**
   Constructs a RefPtrMember with another RefPtrMember.

   If @c rhs is not null and is not a reference to the parent object, addRef() is called
   to increment the reference count.

   @param pParent the parent ManagedObject containing this RefPtrMember as a member
   @param rhs a RefPtrMember pointing to an object of type T, which must be a class
   derived from ManagedObject.
*/
//==============================================================================
template<typename T>
inline
RefPtrMember<T>::RefPtrMember(ManagedObject* pParent, const RefPtrMember<T>& rhs) :
	m_ptr(rhs.get()),
	m_pParent(pParent)
{
	if(m_ptr && (static_cast<ManagedObject*>(m_ptr) != m_pParent))
		m_ptr->addRef();
}

//==============================================================================
// RefPtrMember::RefPtrMember
//
/**
   Constructs a RefPtrMember from a RefPtr for the same type.

   If @c rhs is not null and is not a reference to the parent object, addRef() is called
   to increment the reference count.

   @param pParent the parent ManagedObject containing this RefPtrMember as a member
   @param rhs a RefPtr pointing to an object of type T, which must be a class
   derived from ManagedObject.
*/
// This has been added to give symmetry to the case where a RefPtr can be created from
// a RefPtrMember because of the RefPtrMember conversion operator.  RefPtr
// does not have a conversion operator, so we need this.
//==============================================================================
template<typename T>
inline
RefPtrMember<T>::RefPtrMember(ManagedObject* pParent, const RefPtr<T>& rhs) :
	m_ptr(rhs.get()),
	m_pParent(pParent)
{
	if(m_ptr && (static_cast<ManagedObject*>(m_ptr) != m_pParent))
		m_ptr->addRef();
}

//==============================================================================
// RefPtrMember::operator=
//
/**
   Assigns one RefPtrMember to another.

   If @c rhs contains a non-null object pointer, and is not a reference to
   the parent, addRef() is called to increment the reference count.

   @param rhs the RefPtrMember<T> to copy.
*/
//==============================================================================
template<typename T>
inline
RefPtrMember<T>& RefPtrMember<T>::operator=(const RefPtrMember<T>& rhs)
{
	if(m_ptr != rhs.get())
	{
		// do not release the current ptr until we have addrefed
		// the new value - just in case the new value relies on
		// the existing value for its continued existence
		T* pOld = m_ptr;

		m_ptr = rhs.get();
		if(m_ptr && (static_cast<ManagedObject*>(m_ptr) != m_pParent))
		{
			m_ptr->addRef();
		}
		if(pOld && (static_cast<ManagedObject*>(pOld) != m_pParent))
		{
			pOld->release();
		}
	}
	return *this;
}

//==============================================================================
// RefPtrMember::operator=
//
/**
   Assigns a RefPtr to a RefPtrMember.

   If @c rhs contains a non-null object pointer which is not a reference to
   the parent, addRef() is called to increment the reference count.

   @param rhs the RefPtr<T> to copy.
*/
//==============================================================================
template<typename T>
inline
RefPtrMember<T>& RefPtrMember<T>::operator=(const RefPtr<T>& rhs)
{
	return (*this)=rhs.get();
}

//==============================================================================
// RefPtrMember::operator=
//
/**
   Assigns a pointer.

   If @c ptr is not null, and does not refer to the parent object, addRef() is called 
   to increment the reference count.  If this RefPtrMember previously referenced
   an object, release() is called on that object.

   @param ptr pointer to an object of type T, which must be a class
   derived from ManagedObject.
*/
//==============================================================================
template<typename T>
inline
RefPtrMember<T>& RefPtrMember<T>::operator=(T* ptr)
{
	if(m_ptr != ptr)
	{
		// do not release the current ptr until we have addrefed
		// the new value - just in case the new value relies on
		// the existing value for its continued existence
		T* pOld = m_ptr;
		m_ptr = ptr;
		if(m_ptr && (static_cast<ManagedObject*>(m_ptr) != m_pParent))
		{
			m_ptr->addRef();
		}
		if(pOld && (static_cast<ManagedObject*>(pOld) != m_pParent))
		{
			pOld->release();
		}
	}
	return *this;
}

//==============================================================================
// RefPtrMember::operator<
//
/**
   Less than operator.

   Returns true if this RefPtrMember should be ordered before @c rhs.
   If this and @c rhs both refer to null, @c false is returned.
   @param rhs the RefPtr to compare.
   @since 1.3
*/
//==============================================================================
template<typename T>
inline
bool RefPtrMember<T>::operator<(const RefPtrMember& rhs) const
{
	return (m_ptr < rhs.m_ptr);
}

//==============================================================================
// RefPtrMember::operator==
//
/**
   Equality operator.

   Returns true if this RefPtrMember refers to the same object as @c rhs.
   If this and @c rhs both refer to null, true is returned.
   @param rhs the RefPtrMember to compare.
*/
//==============================================================================
template<typename T>
inline
bool RefPtrMember<T>::operator==(const RefPtrMember& rhs) const
{
	return (m_ptr == rhs.m_ptr);
}

//==============================================================================
// RefPtrMember::operator==
//
/**
   Equality operator.

   Returns true if this RefPtrMember refers to the same object as @c rhs.
   If this and @c rhs both refer to null, true is returned.
   @param rhs the pointer to compare.
*/
//==============================================================================
template<typename T>
inline
bool RefPtrMember<T>::operator==(const T* rhs) const
{
	return (m_ptr == rhs);
}

//==============================================================================
// RefPtrMember::operator!=
//
/**
   Inequality operator.

   Returns true if this RefPtrMember refers to a different object than @c rhs.
   If this and @c rhs both refer to null, false is returned.

   @param rhs the RefPtrMember to compare.
*/
//==============================================================================
template<typename T>
inline
bool RefPtrMember<T>::operator!=(const RefPtrMember& rhs) const
{
	return !(*this == rhs);
}

//==============================================================================
// RefPtrMember::operator!=
//
/**
   Inequality operator.

   Returns true if this RefPtrMember refers to a different object than @c rhs.
   If this and @c rhs both refer to null, false is returned.

   @param rhs the pointer to compare.
*/
//==============================================================================
template<typename T>
inline
bool RefPtrMember<T>::operator!=(const T* rhs) const
{
	return (m_ptr != rhs);
}

//==============================================================================
// RefPtrMember::operator T*
//
/**
   Conversion operator.

   Silently converts this RefPtrMember<T> into a T*.
*/
//==============================================================================
template<typename T>
inline 
RefPtrMember<T>::operator T*() const
{
	return m_ptr;
}

//==============================================================================
// RefPtrMember::isNull
//
/**
   Tests if the contained object pointer is null

   @returns true if the object pointer is null; false otherwise
*/
//==============================================================================
template<typename T>
inline
bool RefPtrMember<T>::isNull() const
{
	return (m_ptr == 0);
}

//==============================================================================
// RefPtrMember::get
//
/**
   Returns the contained object pointer.

   Neither the RefPtrMember nor the reference count of its object are changed.
*/
//==============================================================================
template<typename T>
inline 
T* RefPtrMember<T>::get() const
{
	return m_ptr;
}

//==============================================================================
// RefPtrMember::operator->
//
/**
   A dereferencing operator.  Returns the contained object pointer.
   This operator allows RefPtrMembers to be used as if they were raw C++ pointers.
*/
//==============================================================================
template<typename T>
inline 
T* RefPtrMember<T>::operator->() const
{
	OT_DBG_ASSERT(m_ptr!=0);
	return m_ptr;
}

//==============================================================================
// RefPtrMember::operator*
//
/**
   A dereferencing operator.  Returns the contained object.
   This operator allows RefPtrMembers to be used as if they were raw C++ pointers.
*/
//==============================================================================
template<typename T>
inline 
T& RefPtrMember<T>::operator*() const
{
	OT_DBG_ASSERT(m_ptr!=0);
	return *m_ptr;
}

//==============================================================================
// RefPtrMember::~RefPtrMember
//
/**
   Destructor.  Decrements the reference count of the contained object pointer
   if it is not null and is not the same as the parent object.
*/
//==============================================================================
template<typename T>
inline 
RefPtrMember<T>::~RefPtrMember()
{
	release();
}

//==============================================================================
// RefPtrMember::release
//
/**
   Decrements the reference count of the contained object pointer
   if it is not null and is not the same as the parent object.
   The contained object pointer is then set equal to null.
*/
//==============================================================================
template<typename T>
inline 
void RefPtrMember<T>::release()
{
	if(m_ptr && (static_cast<ManagedObject*>(m_ptr) != m_pParent))
	{
		m_ptr->release();
	}
	m_ptr = 0;
}

OT_BASE_NAMESPACE_END
