//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:42 $
//
//==============================================================================
/**
   @class ot::StringIterator

   Iterator class that will iterate over each Unicode character contained
   within a controlled sequence of ::CharType characters encoded according
   to the internal @OpenTop encoding.  For convenience,
   de-referencing a StringIterator results in a Character object, from
   which the Unicode character can be directly obtained.

   This class implements the required interface for a bidirectional iterator
   as defined by the C++ Standard Library.

   This class enables algorithms from the C++ Standard Library to be used on
   Unicode strings that have been encoded into the internal @OpenTop String encoding.
*/
//==============================================================================

#include "defs.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// StringIterator::StringIterator
//
/**
   Default constructor.
*/
//==============================================================================
inline
StringIterator::StringIterator() :
	m_ptr(0)
{
}

//==============================================================================
// StringIterator::StringIterator
//
/**
   Creates a StringIterator positioned at the first character of 
   the sequence of ::CharType characters starting at @c ptr.
*/
//==============================================================================
inline
StringIterator::StringIterator(const CharType* ptr) :
	m_ptr(ptr)
{
}

//==============================================================================
// StringIterator::operator=
//
/**
   Assignment operator.

   Sets the position of this StringIterator to be the first character of 
   the sequence of ::CharType characters starting at @c ptr.
*/
//==============================================================================
inline
StringIterator& StringIterator::operator=(const CharType* ptr)
{
	m_ptr = ptr;
	return *this;
}

//==============================================================================
// StringIterator::operator==
//
/**
   Equality operator.
   @returns true if this StringIterator is pointing at the same location as
   @c rhs; false otherwise.
*/
//==============================================================================
inline
bool StringIterator::operator==(const StringIterator& rhs) const
{
	return (m_ptr == rhs.m_ptr);
}

//==============================================================================
// StringIterator::operator!=
//
/**
   Inequality operator.
   @returns false if this StringIterator is pointing at the same location as
   @c rhs; true otherwise.
*/
//==============================================================================
inline
bool StringIterator::operator!=(const StringIterator& rhs) const
{
	return (m_ptr != rhs.m_ptr);
}

//==============================================================================
// StringIterator::operator++()
//
/**
   Prefix increment operator.

   Positions the iterator at the next Unicode character in the sequence and
   returns a reference to self.
*/
//==============================================================================
inline
StringIterator& StringIterator::operator++()
{
	if(m_ptr) m_ptr+=SystemCodeConverter::GetCharSequenceLength(*m_ptr);
	return *this;
}

//==============================================================================
// StringIterator::operator++(int)
//
/**
   Postfix increment operator.

   Positions the iterator at the next Unicode character in the sequence and returns
   a copy of self positioned at the original location.
*/
//==============================================================================
inline
StringIterator StringIterator::operator++(int)
{
	const CharType* pOld = m_ptr;
	if(m_ptr) m_ptr+=SystemCodeConverter::GetCharSequenceLength(*m_ptr);
	return pOld;
}

//==============================================================================
// StringIterator::operator--()
//
/**
   Prefix decrement operator.

   Positions the iterator at the previous Unicode character in the sequence and
   returns a reference to self.
*/
//==============================================================================
inline
StringIterator& StringIterator::operator--()
{
	while(m_ptr && SystemCodeConverter::IsSequenceStartChar(*--m_ptr)) {}
	return *this;
}

//==============================================================================
// StringIterator::operator--(int)
//
/**
   Postfix decrement operator.

   Positions the iterator at the previous Unicode character in the sequence and returns
   a copy of self positioned at the original location.
*/
//==============================================================================
inline
StringIterator StringIterator::operator--(int)
{
	const CharType* pOld = m_ptr;
	while(m_ptr && SystemCodeConverter::IsSequenceStartChar(*--m_ptr)) {}
	return pOld;
}

//==============================================================================
// StringIterator::operator*
//
/**
   Dereference operator.

   @returns the Unicode character represented by the ::CharType sequence starting
   at the current location of the iterator.
*/
//==============================================================================
inline
Character StringIterator::operator*() const
{
	// Note: As we do not know the size of the charcater array,
	// We have to invent a length for the Character constructor.

	return Character(m_ptr, 6 /* any number >= utf-8 max len */);
}


//==============================================================================
// StringIterator::data()
//
/**
   Returns a pointer to the current position within the ::CharType sequence.

   @returns a pointer to the current data position of this iterator.
*/
//==============================================================================
inline
const CharType* StringIterator::data() const
{
	return m_ptr;
}

OT_BASE_NAMESPACE_END
