//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:45 $
//
//==============================================================================
//
// Inline members for SystemCodeConverter
//
//==============================================================================

#ifndef OT_BASE_defs_h
#include "defs.h"
#endif //OT_BASE_defs_h
#include "CodeConverterBase.h"
#include "String.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// SystemCodeConverter::GetCharSequenceLength
//
/**
   Returns the number of ::CharType characters that make up a multi-character
   sequence.

   Unless the internal encoding used by the library is UCS-4 (where ::CharType
   is at least 21-bits wide and can represent all Unicode characters from 
   U+0000 - U+10FFFF), Unicode characters are represented internally using a sequence
   of one or more ::CharType characters.
    
   Each multi-character encoding supported (UTF-16 and UTF-8) allows the length 
   of the sequence to be determined from the first character of the sequence.

   In the case of UTF-16 this is 1 unless @c ch is a surrogate pair start character
   (0xD800-0xDBFF) in which case the length is 2.
   
   In the case of UTF-8 the sequence length can be established by looking at the
   number of high-order  bits set to '1' in the passed char @c ch.  If no high-order 
   bits are set, then the passed character is equivalent to an ASCII character and
   the sequence has a length of 1.  In common with the rest of @OpenTop, this method
   does not recognize UTF-8 sequences greater than 4 bytes.  Lead bytes that indicate
   sequences longer than 4 are treated as indicating a sequence of length 1.

   @returns the length of the multi-character sequence denoted by the start
            character @c ch
   @param ch The first character in a sequence of one or more ::CharType
            characters
*/
//==============================================================================
inline 
size_t SystemCodeConverter::GetCharSequenceLength(UCharType ch)
{

#if defined(OT_UTF8)

	return s_TrailingBytesForUTF8[ch] + 1;

#elif defined(OT_UTF16)

	if((ch & 0xF800) == 0xD800) // first part of surrogate pair
		return 2;
	else
		return 1;

#elif defined(OT_UCS4) || defined(OT_UCS2)

	return 1;

#else
	#error Internal character encoding not correctly specified
	return 1;

#endif

}

//==============================================================================
// SystemCodeConverter::IsSequenceStartChar
//
/**
   Tests the passed character @c ch to see if it marks the start of
   a multi-character sequence, a standalone character or a trailing character.

   @param ch character to test
   @returns true if @c ch is either a standalone character or marks the start
            of a multi-character sequence; false otherwise

*/
//==============================================================================
inline
bool SystemCodeConverter::IsSequenceStartChar(UCharType ch)
{
#if defined(OT_UTF8)

	// valid sequence start are ascii chars (<0x80) or
	// >= 0xC0

	return (ch < 0x80U || ch > 0xBFU);

#elif defined(OT_UTF16)

	//
	// Simply must not be in the 2nd half of a surrogate pair
	//
	// The 2nd half of a surrogate pair is in the range 0xDC00 - 0xDFFF
	// 0xDC00 = 1101 1100 0000 0000
	// 0xDFFF = 1101 1111 1111 1111
	// mask     1111 1100 0000 0000 = 0xFC00

	return ((ch & 0xFC00) != 0xDC00);

#elif defined(OT_UCS4) || defined(OT_UCS2)

	return true;

#else
	#error Internal character encoding not correctly specified
	return false;

#endif
}

//==============================================================================
// SystemCodeConverter::IsValidCharSequence
//
/**
   Tests the passed ::CharType sequence @c from, for a length of
   @c len to see if it represents a properly encoded Unicode character.

   @param from pointer to the first ::CharType character in the sequence
   @param len the number of ::CharType characters in the encoded sequence
   @returns true if the sequence represents a valid Unicode character; false otherwise
*/
//==============================================================================
inline
bool SystemCodeConverter::IsValidCharSequence(const CharType* from, size_t len)
{

#if defined(OT_UTF8)

	return IsLegalUTF8((const Byte*)from, len);

#elif defined(OT_UTF16)

	return IsLegalUTF16((const wchar_t*)from, len);

#elif defined(OT_UCS4)

	return (*from <= 0x10FFFF);

#else

	return true;

#endif

}

OT_BASE_NAMESPACE_END
