//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.3 $
// $Date: 2003/01/25 12:00:55 $
//
//==============================================================================
//
// Class: Thread
// 
//==============================================================================

#ifndef OT_BASE_Thread_h
#define OT_BASE_Thread_h

#ifndef OT_BASE_defs_h
#include "defs.h"
#endif //OT_BASE_defs_h

#include "Runnable.h"
#include "AtomicCounter.h"
#include "ConditionVariable.h"
#include "FastMutex.h"
#include "RefPtr.h"
#include "RecursiveMutex.h"
#include "String.h"
#include "Monitor.h"
#include "ThreadLocal.h"
#include "ThreadId.h"

#include <list>

OT_BASE_NAMESPACE_BEGIN

#ifndef OT_DOCUMENTATION_ONLY
extern "C" void* OpenTopThreadFunc(void*);
#endif //OT_DOCUMENTATION_ONLY

class OT_BASE_PKG Thread : public Runnable, public Monitor
{
	friend class System;
	friend void* OpenTopThreadFunc(void*);

public: // static functions available in single-threaded versions

	static void Sleep(long millis);
	static void Sleep(long millis, long nanos);

	//
	// Everything from here is only present in multi-threaded
	// versions of the library
	//
#ifdef OT_MT

protected: // constructors for use by derived classes

	Thread();
	Thread(const String& name);

public:
	typedef std::list< RefPtr<Thread> > ThreadList;
	enum { MinPriority=1, NormPriority=5, MaxPriority=10 };

	Thread(Runnable* pTarget);
	Thread(Runnable* pTarget, const String& name);

	~Thread();

	ThreadId getId() const;
	String getName() const;
	unsigned getPriority() const;

	bool isActive() const;
	bool isDaemon() const;

	void join();
	void join(long millis);

	void setPriority(unsigned priority);
	void setDaemon(bool bDaemon);
	void setName(const String& name);
	void interrupt();
	bool interruptSupported() const;

	void start();
	
	bool operator==(const Thread& rhs) const;
	bool operator!=(const Thread& rhs) const;

	// From Runnable...
	virtual void run();

#if defined(OT_WIN32_THREADS)
	HANDLE getWin32Handle() const;
#endif

public: // static functions

	static void Yield();
	static RefPtr<Thread> CurrentThread();
	static ThreadId CurrentThreadId();
	static ThreadList GetActiveThreads();
	static size_t GetActiveCount();
	static int GetInterruptSignal();
	static void SetInterruptSignal(int signo);

private: // static functions
	static String GenerateName();
	static void TerminateAllDaemonThreads();
	static void WaitAllUserThreads();

private:
	enum State {Initial, Active, Terminated};
	void setState(State state);
	void init();
	void setExecutionPriority();
	void cancel();
	void doRun();

private:
	State m_state;
	String m_name;
	bool m_bDaemon;
	const RefPtr<Runnable> m_rpRunnable;
	ThreadId m_threadId;
	unsigned m_priority;

#if defined(OT_WIN32_THREADS)
	HANDLE m_hThread;
#elif defined(OT_POSIX_THREADS)
	bool m_bJoined;
#endif

	static AtomicCounter s_nextThreadNumber;
	static ThreadLocal s_thisPointer;
	static ThreadList s_activeThreadList;
	static int s_interruptSignal;

#endif //OT_MT

};

OT_BASE_NAMESPACE_END

#endif //OT_BASE_Thread_h
