//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:48 $
//
//==============================================================================
//
// Inline members for UnicodeCharacterType
//
//=============================================================================
// Class UnicodeCharacterType
/**
	@class ot::UnicodeCharacterType
	
	@brief Class module for determining the characteristics of a given
	Unicode character.

    This class mirrors some of the facilities provided by the
	standard C++ library.  However, care must be taken to ensure that a Unicode
	locale is being used before relying on these.  The motivations for the existence
	of this class are:
	- to handle non-standard C++ implementations
	- to cater for C++ implementations where there is no Unicode support
*/
//==============================================================================

#ifndef OT_BASE_defs_h
#include "defs.h"
#endif //OT_BASE_defs_h

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// UnicodeCharacterType::IsAscii
//
/**
   Tests if the Unicode character represented by the single ::CharType @c ch
   is an ASCII character (0-0x7F).

   @returns true if this Character is an ASCII character; false otherwise.
*/
//==============================================================================
inline
bool UnicodeCharacterType::IsAscii(UCharType ch)
{
	return (ch <= 0x7F);
}

//==============================================================================
// UnicodeCharacterType::IsSpace
//
/**
   Tests if the Unicode character represented by the single ::CharType @c ch
   is white-space according to common Windows and Unix conventions.

   Space characters are:-
   - <tt>'\t' U+0009 HORIZONTAL TABULATION</tt>
   - <tt>'\\n' U+000A NEW LINE</tt>
   - <tt>'\f' U+000C FORM FEED</tt> 
   - <tt>'\r' U+000D CARRIAGE RETURN</tt>  
   - <tt>' '   U+0020 SPACE</tt> 

   @returns true if @c ch is a space character; false otherwise.
*/
//==============================================================================
inline
bool UnicodeCharacterType::IsSpace(UCharType ch)
{
	switch(ch)
	{
		case 0x0009:
		case 0x000A:
		case 0x000C:
		case 0x000D:
		case 0x0020:
			return true;
		default:
			return false;
	}
}

//==============================================================================
// UnicodeCharacterType::IsDigit
//
/**
   Tests if the Unicode character represented by the single ::CharType @c ch
   is an ASCII decimal digit 0-9.

   @returns true if this Character is a decimal digit [0-9]; false otherwise.
*/
//==============================================================================
inline
bool UnicodeCharacterType::IsDigit(UCharType ch)
{
	return (ch >= '0' && ch <= '9');
}

//==============================================================================
// UnicodeCharacterType::IsHexDigit
//
/**
   Tests if the Unicode character represented by the single ::CharType @c ch
   is an ASCII hexadecimal digit [0-9], [A-F], [a-f].

   @returns true if this Character is a hexadecimal digit; false otherwise.
*/
//==============================================================================
inline
bool UnicodeCharacterType::IsHexDigit(UCharType ch)
{
	return ((ch >= '0' && ch <= '9') ||
	        (ch >= 'a' && ch <= 'f') ||
	        (ch >= 'A' && ch <= 'F'));
}

//==============================================================================
// UnicodeCharacterType::IsAlphaNumeric
//
/**
   Tests if the Unicode character represented by the single ::CharType @c ch
   is an alpha-numeric character[a-z], [A-Z], [0-9].

   @returns true if this Character is an alpha-numeric character; false otherwise.
*/
//==============================================================================
inline
bool UnicodeCharacterType::IsAlphaNumeric(UCharType ch)
{
	return ((ch >= '0' && ch <= '9') ||
	        (ch >= 'a' && ch <= 'z') ||
		    (ch >= 'A' && ch <= 'Z'));
}

OT_BASE_NAMESPACE_END
