//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:12 $
//
//==============================================================================

#ifndef OT_IO_defs_h
#include "defs.h"
#endif //OT_IO_defs_h

OT_IO_NAMESPACE_BEGIN

//==============================================================================
// Console::cout
//
/**
   Returns a raw C++ reference to the PrintWriter which is connected
   to the @c stdout file handle.

   This function has been provided to make it more convenient to use
   the C++ IO operators on the standard output streams.

   @code
   int age = 5;
   Console::cout() << OT_T("I am ") << age
                   << OT_T(" years old") << endl;
   @endcode
  
   It is unusual for a @OpenTop function to return a raw C++ reference
   to a ManagedObject.  However, in this case, @OpenTop will ensure that
   the PrintWriter object exists for the lifetime of the application,
   thereby removing the need for application code to maintain
   the reference count.

   This function is also remarkable because it deviates from the @OpenTop
   convention of starting static member functions with upper case.  This
   is to be consistent with the standard C++ @c ostream called @c cout.
*/
//==============================================================================
inline PrintWriter& Console::cout()
{
	return *Out().get();
}

//==============================================================================
// Console::cerr
//
/**
   Returns a raw C++ reference to the PrintWriter which is connected
   to the @c stderr file handle.

   This function has been provided to make it more convenient to use
   the C++ IO operators on the standard output streams.

   @code
   try
   {
       ...
   }
   catch(Exception& e)
   {
       Console::cerr() << e.toString() << endl;
   }
   @endcode
  
   It is unusual for a @OpenTop function to return a raw C++ reference
   to a ManagedObject.  However, in this case, @OpenTop will ensure that
   the PrintWriter object exists for the lifetime of the application,
   thereby removing the need for application code to maintain
   the reference count.

   This function is also remarkable because it deviates from the @OpenTop
   convention of starting static member functions with upper case.  This
   is to be consistent with the standard C++ ostream called @c cerr.
*/
//==============================================================================
inline PrintWriter& Console::cerr()
{
	return *Err().get();
}

//==============================================================================
// Console::cin
//
/**
   Returns a raw C++ reference to the InputStreamReader which is connected
   to the @c stdin file handle.

   It is unusual for a @OpenTop function to return a raw C++ reference
   to a ManagedObject.  However, in this case, @OpenTop will ensure that
   the InputStreamReader object exists for the lifetime of the application,
   thereby removing the need for application code to maintain
   the reference count.

   This function is also remarkable because it deviates from the @OpenTop
   convention of starting static member functions with upper case.  This
   is to be consistent with the standard C++ ostream called @c cin.
*/
//==============================================================================
inline InputStreamReader& Console::cin()
{
	return *In().get();
}

OT_IO_NAMESPACE_END
