//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:50 $
//
//==============================================================================
//
// Class Date
//
// Class DateTime
//
// This class is used to represent a precise point in time (with an accuracy
// of one millisecond) without reference to a particular calendar system.
//
// Internally, the date and time is represented by two integer values, one
// representing the Julian Day number (number of days ellapsed from a reference
// day) and another representing the number of milliseconds since midnight.
//
// The Gregorian Callendar
// -----------------------
// In accordance with common practice, this class provides interfaces
// to set and extract date information using the Gregorian Calendar notations
// of year, month and day.  When these interfaces are used, the DateTime class
// convertes between its internal Julian Day representation to the Gregorian 
// callendar.
// This class can be used to store any date in any calendar system, but other
// calendar systems will have to make use of the internal representation.
//
//==============================================================================

#ifndef OT_UTIL_DateTime_h
#define OT_UTIL_DateTime_h

#ifndef OT_UTIL_defs_h
#include "defs.h"
#endif //OT_UTIL_defs_h

#include <time.h>

OT_UTIL_NAMESPACE_BEGIN

class OT_UTIL_PKG DateTime
{
public:

	DateTime(); // construct an invalid DateTime
	DateTime(UShort year, UShort month, UShort day);
	DateTime(UShort year, UShort month, UShort day, UShort hour, UShort minute, UShort second, UShort ms);
	DateTime(const String& strDateTime);
	DateTime(unsigned long julianDay);
	DateTime(unsigned long julianDay, unsigned long milliseconds);

	bool operator==(const DateTime& rhs) const;
	bool operator!=(const DateTime& rhs) const;
	bool operator<(const DateTime& rhs) const;
	bool operator<=(const DateTime& rhs) const;
	bool operator>(const DateTime& rhs) const;
	bool operator>=(const DateTime& rhs) const;

	unsigned long getJulianDay() const;
	unsigned long getTimeOfDay() const;
	time_t toAnsiTime() const; //todo: make consistent with FromAnsiTime
	void toAnsiTM(struct tm* ptm) const;
	
	void getDate(UShort& year, UShort& month, UShort& day, UShort& dayOfWeek) const;
	void getTime(UShort& hour, UShort& minute, UShort& second, UShort& millisecond) const;
	void setDate(UShort year, UShort month, UShort day);
	void setTime(UShort hour, UShort minute, UShort second, UShort millisecond);
	void adjust(short days, short hours, short minutes, short seconds, short milliseconds);
	bool isValid() const;
	String toString() const;
	String Format(const String& format) const;

	static bool IsValidDate(UShort year, UShort month, UShort day);
	static bool IsValidTime(UShort hour, UShort minute, UShort second, UShort ms);
	static bool IsLeapYear(UShort nYear);
	
	static DateTime GetSystemTime();
	static DateTime FromAnsiTime(long secsSinceEpoch, unsigned long microSeconds);

protected:
	bool parseDateTime(const String& strDate);

private:
	unsigned long m_JD;
	unsigned long m_ms;
};

OT_UTIL_NAMESPACE_END

#endif //OT_UTIL_MIMEType_h
