//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:50 $
//
//==============================================================================
//
// This class is based upon CountedPtr from the book 
// "The C++ Standard Library" by Nicolai M Josuttis
//
// It implements a non-invasive reference counting scheme
// where the count is maintained seperately from the object
// that is reference counted.
//
//==============================================================================

#ifndef OT_UTIL_RefCountedPtr_h
#define OT_UTIL_RefCountedPtr_h

#ifndef OT_UTIL_defs_h
#include "defs.h"
#endif //OT_UTIL_defs_h

OT_UTIL_NAMESPACE_BEGIN

template <typename T>
class RefCountedPtr
{
public:
	// constructor taking value returned from new()
	explicit RefCountedPtr(T* ptr=NULL) :
		m_pObject(ptr), m_pCount(new long(1)) {}

	// copy constructor - one more owner
	RefCountedPtr(const RefCountedPtr<T>& rhs) :
		m_pObject(rhs.m_pObject), m_pCount(rhs.m_pCount)
	{
		++*m_pCount;
	}

	// destructor - delete valuer (and count) if this is the last owner
	~RefCountedPtr()
	{
		dispose();
	}

	// assignment (release old and share new value)
	RefCountedPtr<T>& operator=(const RefCountedPtr<T>& rhs)
	{
		if (this != &rhs)
		{
			dispose();
			m_pObject = rhs.m_pObject;
			m_pCount = rhs.m_pCount;
			++*m_pCount;
		}
		return *this;
	}

	// accessors
	T& operator*() const
	{
		return *m_pObject;
	}

	T* operator->() const
	{
		return m_pObject;
	}

	T* get() const
	{
		return m_pObject;
	}

private:
	void dispose()
	{
		if(--*m_pCount == 0)
		{
			delete m_pCount;
			delete m_pObject;
		}
	}

	T* m_pObject;
	long* m_pCount;
};

OT_UTIL_NAMESPACE_END

#endif //OT_UTIL_RefCountedPtr_h

