//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:26:07 $
//
//==============================================================================
//
// Class: Scanner
// 
// Overview
// --------
// Provides the mechanisms to 
// i)   read an input stream
// ii)  buffer the stream data and provide direct and efficient access to
//      any previously read location.  (The stream is read into buffers which
//      are reference-counted, the buffers remain available for so long as a 
//      reference is held by the application - normally via a ScannerPosition
//      object)
// iii) provide customized look-ahead routines to determine the contents of
//      the stream at a given position - making use of the character masks
//      from the CharTypeFacet class.
//
//==============================================================================

#ifndef OT_XML_Scanner_h
#define OT_XML_Scanner_h

#ifndef OT_XML_defs_h
#include "defs.h"
#endif //OT_XML_defs_h

#ifndef OT_XML_ScannerPosition_h
#include "ScannerPosition.h"
#endif //OT_XML_ScannerPosition_h

#ifndef OT_XML_CharTypeFacet_h
#include "CharTypeFacet.h"
#endif //OT_XML_CharTypeFacet_h

OT_XML_NAMESPACE_BEGIN

class Buffer;
class ExternalEntity;
class StreamPosition;
class BufferRange;

class OT_XML_PKG Scanner
{
public:

	static ScannerPosition GetPosition(const Entity& entity,
	                                   const StreamPosition& location,
	                                   size_t bufferSize=0);

	static bool PeekNextStringConstant(ScannerPosition& position, const String& str);
	static bool PeekNextStringConstant(ScannerPosition& position, const char* pStr);

	static Character PeekNextCharacter(const ScannerPosition& position);
	static Character PeekNextCharacterImpl(const ScannerPosition& position);

	static String PeekNextContigString(ScannerPosition& position,
	                                   CharTypeFacet::Mask includeMask,
	                                   CharTypeFacet::Mask excludeMask);

	static Character GetNextCharacter(ScannerPosition& position);
	static Character GetNextCharacterImpl(ScannerPosition& position);

	static BufferRange GetNextContigString(ScannerPosition& position,
	                                       CharTypeFacet::Mask includeMask,
	                                       CharTypeFacet::Mask excludeMask,
										   const String& delim,
	                                       Character& nextChar,
	                                       size_t maxSize=0);

	static BufferRange GetEntityBuffer(const ScannerPosition& position);

	static String GetNextStringDelimited(ScannerPosition& position,
	                                     CharTypeFacet::Mask includeMask,
	                                     CharTypeFacet::Mask excludeMask,
	                                     CharType delimiter,
								         Character& nextChar,
										 size_t maxSize=0);

	static String GetNextStringDelimited(ScannerPosition& position,
	                                     CharTypeFacet::Mask includeMask,
	                                     CharTypeFacet::Mask excludeMask,
	                                     const String& delimiter,
								         Character& nextChar);

	static String GetNextContigDecimalString(ScannerPosition& position,
	                                         bool bAllowDecimalPoint,
	                                         Character& nextChar);

	static String GetNextContigHexString(ScannerPosition& position,
	                                     Character& nextChar);

	static size_t SkipWhiteSpace(ScannerPosition& position);
	
	static bool SkipSoftEOF(ScannerPosition& position);

	static void Skip(ScannerPosition& position, size_t n);
	
	static CharType SkipToDelimiter(ScannerPosition& position,
	                                CharType delim, CharTypeFacet::Mask includeMask);
	
	static CharType SkipToDelimiter(ScannerPosition& position,
	                                const String& delim, CharTypeFacet::Mask includeMask);
	
	static int SkipToDelimiters(ScannerPosition& position, size_t numDelimiters,
	                            const char* delims[]);
	
	static bool SkipNextStringConstant(ScannerPosition& position, const String& str);
	static bool SkipNextStringConstant(ScannerPosition& position, const char* pStr);
	
	static bool SkipNextCharConstant(ScannerPosition& position, CharType x);
	
	static ScannerPosition BackTrackToChar(const ScannerPosition& position, CharType toChar, size_t maxLen);

	static const ExternalEntity* GetCurrentExternalEntity(const ScannerPosition& position);

protected:
	static Buffer* ReadAndChainNextBuffer(Entity* pEntity, Buffer* pOldBuffer=NULL);

private:
	Scanner(); // not implemented
};

/*
inline CharType Scanner::GetNextChar(ScannerPosition& position)
{
	Buffer* pBuffer = position.m_rpBuffer.get();
	if (position.m_bufferOffset < pBuffer->m_used)
	{
		const CharType ret = pBuffer->m_pData[position.m_bufferOffset++];
		position.m_streamPosition.incrementByChar(ret);
		return ret;
	}
	else
	{
		return GetNextCharImpl(position);
	}
}
*/
inline Character Scanner::GetNextCharacter(ScannerPosition& position)
{
	Buffer* pBuffer = position.m_rpBuffer.get();
	const size_t charsAvailable = (pBuffer->m_used - position.m_bufferOffset);
	if(charsAvailable)
	{
		const Character ret(pBuffer->m_pData+position.m_bufferOffset, charsAvailable);
		position.m_bufferOffset += ret.length();
		position.m_streamPosition.incrementByChar(ret);
		return ret;
	}
	else
	{
		return GetNextCharacterImpl(position);
	}
}

inline Character Scanner::PeekNextCharacter(const ScannerPosition& position)
{
	Buffer* pBuffer = position.m_rpBuffer.get();
	const size_t charsAvailable = (pBuffer->m_used - position.m_bufferOffset);
	if(charsAvailable)
	{
		return Character(pBuffer->m_pData+position.m_bufferOffset, charsAvailable);
	}
	else
	{
		return PeekNextCharacterImpl(position);
	}
}

OT_XML_NAMESPACE_END

#endif //OT_XML_Scanner_h
