//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:26:07 $
//
//==============================================================================
//
// ScannerPosition
//
// Used to record the position within a scanner's input stream.  An instance
// of this class is passed to the scanner for each scanning operation.
// By recording position outside of the scanner class itself, the parsing
// application is given the power to back-track to any position in the input
// stream.
//
// The ScannerPosition owns a pointer to the "next" position.  This caters for
// entity expansion by inserting lexical tokens into the current position of
// the input stream.
// 
//==============================================================================

#ifndef OT_XML_ScannerPosition_h
#define OT_XML_ScannerPosition_h

#ifndef OT_XML_defs_h
#include "defs.h"
#endif //OT_XML_defs_h

#include "Position.h"
#include "Buffer.h"
#include "StreamPosition.h"

OT_XML_NAMESPACE_BEGIN

class StreamPosition;
class Entity;

class OT_XML_PKG ScannerPosition : public Position
{
	friend class Scanner;

public:

	enum EOFAction {softEOF, skip, space};

	ScannerPosition() : m_pNext(NULL), m_eofAction(softEOF) {};
	virtual ~ScannerPosition();
	ScannerPosition(const ScannerPosition& rhs);
	ScannerPosition& operator=(const ScannerPosition& rhs);

	void setNextPosition(const ScannerPosition& next, EOFAction eofAction);
	const Entity& getEntity() const;
	const StreamPosition& getStreamPosition() const;
	bool isValid() const;

	//
	// Position methods
	//
	virtual size_t getColumnNumber() const;
	virtual size_t getLineNumber() const;
	virtual size_t getOffset() const;
	virtual String getPublicId() const;
	virtual String getSystemId() const;
	virtual String getResolvedSystemId() const;

private:

	ScannerPosition(Buffer* pBuffer);
	ScannerPosition(Buffer* pBuffer, const StreamPosition& streamPosition);

private:
	ScannerPosition* m_pNext;
	RefPtr<Buffer> m_rpBuffer;
	size_t m_bufferOffset;
	StreamPosition m_streamPosition;
	EOFAction m_eofAction;
};

inline ScannerPosition::~ScannerPosition()
{
	delete m_pNext;
}

inline ScannerPosition::ScannerPosition(const ScannerPosition& rhs) :
	m_pNext(NULL)
{
	*this = rhs;
}

inline ScannerPosition& ScannerPosition::operator=(const ScannerPosition& rhs)
{
	if(this != &rhs)
	{
		m_streamPosition = rhs.m_streamPosition;
		m_bufferOffset = rhs.m_bufferOffset;
		m_rpBuffer = rhs.m_rpBuffer;
		m_eofAction = rhs.m_eofAction;

		ScannerPosition* pNewNext = NULL;
		if(rhs.m_pNext)
		{
			pNewNext = new ScannerPosition(*rhs.m_pNext);
		}
		delete m_pNext;
		m_pNext = pNewNext;
	}
	return *this;
}


OT_XML_NAMESPACE_END

#endif //OT_XML_ScannerPosition_h
